//*****************************************************************************
//-----------------------------------------------------------------------------
// Fast simulation and reconstruction package for TOP counter (F77 core)
// M. Staric, Oct-2009
//-----------------------------------------------------------------------------
//
// get_pdf.cc
// example of how to get PDF: 
//    PDF's of pion and kaon mass hypotheses are stored in four 2D 
//    histograms time (0:50ns) versus channel (0:63)
//-----------------------------------------------------------------------------
//*****************************************************************************

#include <iostream>
#include <iomanip>
#include <math.h>
#include "TOPreco.h"
#include "TOPutil.h"
#include "RandomParticle.h"
#include "TOPhbook.h"

void TOPconfigure(int geo, int PMTchoice, int QEchoice, double Redge);
double Masses[5]={.511E-3, .10566, .13957, .49368, .93827};
int Nhyp=5;

int main()
{

  HbookOpen("../hbo/pdf.hbook");
  hbook2(100,"pion PDF, iy=0",64,0,64,1000,0,50);
  hbook2(101,"pion PDF, iy=1",64,0,64,1000,0,50);
  hbook2(102,"pion PDF, iy=2",64,0,64,1000,0,50);
  hbook2(103,"pion PDF, iy=3",64,0,64,1000,0,50);
  hbook2(200,"kaon PDF, iy=0",64,0,64,1000,0,50);
  hbook2(201,"kaon PDF, iy=1",64,0,64,1000,0,50);
  hbook2(202,"kaon PDF, iy=2",64,0,64,1000,0,50);
  hbook2(203,"kaon PDF, iy=3",64,0,64,1000,0,50);

  TOPconfigure(1,1,1,0);
  TOPreco reco(Nhyp, Masses);
  RandomParticle p(2, 60, 5); // 2 GeV/c pion from IP at theta=60deg., phi=5deg.

  reco.Clear();
  reco.Reconstruct(p);
  double r[3], dir[3], tlen, tof, mom; int QbarID;
  reco.GetHit(Local, r, dir, tlen, tof, mom, QbarID); // need QbarID only
  int LR=0; // left-side exit window

  // pion PDF
  for(int ipmt=0; ipmt<16; ipmt++){
    for(int ix=0; ix<4; ix++){
      for(int iy=0; iy<4; iy++){
	int ch=chID(ix, iy, ipmt, 0, LR, QbarID);
	for(double t=25e-3; t<50; t+=50e-3){
	  double pdf=reco.PDF(ch, t, Masses[2]);
	  hfill(100+iy, double(ix+4*ipmt), t, pdf);
	}
      }
    }
  }

  // kaon PDF
  for(int ipmt=0; ipmt<16; ipmt++){
    for(int ix=0; ix<4; ix++){
      for(int iy=0; iy<4; iy++){
	int ch=chID(ix, iy, ipmt, 0, LR, QbarID);
	for(double t=25e-3; t<50; t+=50e-3){
	  double pdf=reco.PDF(ch, t, Masses[3]);
	  hfill(200+iy, double(ix+4*ipmt), t, pdf);
	}
      }
    }
  }
  
  HbookClose();
  return 0;
}

