//*****************************************************************************
//-----------------------------------------------------------------------------
// Fast simulation and reconstruction package for TOP counter (F77 core)
// M. Staric, Sept-2010
//-----------------------------------------------------------------------------
//
// TOPparticle.h
// C++ interface to F77 functions: MC particle
//-----------------------------------------------------------------------------
//*****************************************************************************

#ifndef _TOPparticle_h
#define _TOPparticle_h

#include "RandomParticle.h"
#include <math.h>

class TOPparticle
{
 public:
  // constructors
  TOPparticle();
  TOPparticle(double x, double y, double z, double Px, double Py, double Pz, 
	      double Tof, int Lund, int label=0);
  TOPparticle(RandomParticle p);
  // selectors
  double X(){return m_X;} 
  double Y(){return m_Y;} 
  double Z(){return m_Z;} 
  double Px(){return m_Px;} 
  double Py(){return m_Py;} 
  double Pz(){return m_Pz;} 
  double Tof(){return m_Tof;} 
  double beta(){return m_p/sqrt(m_p*m_p+m_mass*m_mass);}
  double Tlen(){return Tof()*beta()*29.9798458;}
  int Lund(){return m_LUND;} 
  int Charge(){return m_Q;}
  int Q(){return m_Q;}
  double mass(){return m_mass;}
  double p(){return m_p;};
  double theta(){return acos(m_Pz/m_p);}
  double phi(){return atan2(m_Py,m_Px);}
  double Rho(){return sqrt(m_X*m_X+m_Y*m_Y);}
  double Phi(){return atan2(m_Y,m_X);}
  int Hyp();  // 1=e, 2=mu, 3=pi, 4=K, 5=p, 0=other
  int Label(){return m_REF;}
  // modifiers
  void setMass(double mass){m_mass=mass;}
  void setCharge(int Q){m_Q=Q;}
  void swim(double Dist); // move particle along it's track by distance Dist
  bool toCylinder(double Rho); // move particle to radius Rho (back, if outof)
  // print to std output
  void Dump();

 private:
  double m_X;    // point
  double m_Y;    // point
  double m_Z;    // point
  double m_Px;   // momentum
  double m_Py;   // momentum
  double m_Pz;   // momentum
  double m_Tof;  // time-of-flight
  int m_LUND;    // LUND code
  int m_Q;       // charge
  double m_mass; // mass
  double m_p;    // momentum     
  int m_REF;     // reference label (optional)
  void initMem(double x, double y, double z, 
	       double Px, double Py, double Pz, 
	       double Tof, int Lund, int label);
};

#endif

