//*****************************************************************************
//-----------------------------------------------------------------------------
// Fast simulation and reconstruction package for TOP counter (F77 core)
// M. Staric, March-2009
//-----------------------------------------------------------------------------
//
// TOPutil.h
// C++ interface to F77 functions: data conversion utility (for debugging etc.)
// units: cm, ns
//-----------------------------------------------------------------------------
//*****************************************************************************

#ifndef _TOPutil_h
#define _TOPutil_h

#include "TOPf77fun.h"

// returns channel ID from position in local frame of QbarID
inline int chID(double X, double Y, double Z, int QbarID)
{
  float x=X; float y=Y; float z=Z;
  return ich_digiz_(&x, &y, &z, &QbarID);
}

// returns channel ID from position in local frame of QbarID
inline int chID(double X, double Y, int LR, int QbarID)
{
  float x=X; float y=Y;
  return ich_digi_(&x, &y, &LR, &QbarID);
}

// returns channel ID from PMTchannel, PMTid, window number, QbarID 
inline int chID(int IX, int IY, int PMTidX, int PMTidY, int LR, int QbarID)
{
  return ichan_(&IX, &IY, &PMTidX, &PMTidY, &LR, &QbarID);
}

// converts channel ID of QbarID to PMTchannel, PMTid, window number
inline void chIDtoIII(int chID, int QbarID, 
		      int & IX, int & IY, int & PMTidX, int & PMTidY, int & LR)
{
  ichiii_(&chID, &QbarID, &IX, &IY, &PMTidX, &PMTidY, &LR);
}

// converts channel ID to position in local frame of QbarID
inline void chIDtoXYZ(int chID, int QbarID, 
		      double & X, double & Y, double & Z)
{
  float x, y, z;
  ichxyz_(&chID, &QbarID, &x, &y, &z);
  X=x; Y=y; Z=z;
}

// returns digitised time
inline int TDCdigi(double T)
{
  float t=T;
  return tdc_digi_(&t);
}

// returns time in nanoseconds from digitised value
inline double TIME(int TDC)
{
  return tdc_to_t_(&TDC);
}

// transformation of 3D point from Belle to Qbar frame
inline void PointToLocal(int QbarID, double X, double Y, double Z, 
			 double & x, double & y, double & z)
{
  float r[3]={X, Y, Z};
  float dir[3]={0, 0, 0};
  to_local_(r, dir, &QbarID);
  x=r[0]; y=r[1]; z=r[2];
}

// transformation of 3D point from Qbar to Belle frame
inline void PointToGlobal(int QbarID, double X, double Y, double Z, 
			  double & x, double & y, double & z)
{
  float r[3]={X, Y, Z};
  float dir[3]={0, 0, 0};
  to_global_(r, dir, &QbarID);
  x=r[0]; y=r[1]; z=r[2];
}

// transformation of momentum vector from Belle to Qbar frame
inline void MomentumToLocal(int QbarID, double Px, double Py, double Pz, 
			    double & px, double & py, double & pz)
{
  float r[3]={0, 0, 0};
  float dir[3]={Px, Py, Pz};
  to_local_(r, dir, &QbarID);
  px=dir[0]; py=dir[1]; pz=dir[2];
}

// transformation of momentum vector from Qbar to Belle frame
inline void MomentumToGlobal(int QbarID, double Px, double Py, double Pz, 
			     double & px, double & py, double & pz)
{
  float r[3]={0, 0, 0};
  float dir[3]={Px, Py, Pz};
  to_global_(r, dir, &QbarID);
  px=dir[0]; py=dir[1]; pz=dir[2];
}

#endif

